function inbinom( pnb, r, p, ier )

!-----------------------------------------------------------------------
!
!     Inverse of the negative binomial cdf.
!     Given that a random variable is distributed according to the
!     negative binomial distribution with R successes and probability P,
!     it returns the smallest integer value such that the probability
!     that the random variable is less than or equal to that value is
!     at least PNB.
!
!     PNB   - Input . Input probability (cdf) (0<= PNB <=1)   - Real
!     R     - Input . Number of successes (R > 0)             - Integer
!     P     - Input . Probability of success (0<= P <=1)      - Real
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                     3 = error in IBETA
!
!     External functions called:
!       NBINOMCDF
!
!     Fortran functions called:
!       NONE
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   integer :: inbinom

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: pnb, p
   integer, intent(in) :: r
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

   real(kind=8), external :: nbinomcdf

   real(kind=8), parameter :: zero=0.0_8, one=1.0_8
   real(kind=8) :: t
   integer :: i

   !--------------------------------------------------------------------

   inbinom = -1
   ier = 1

   !  Test for valid input arguments

   if ( r <= 0 .or. pnb < zero .or. pnb > one      &
               .or. p < zero .or. p > one ) return

   i = 0
   do
      t = nbinomcdf( i, r, p, ier )
      if ( ier /= 0 ) return
      if ( t >= pnb ) exit
      i = i + 1
   end do
   inbinom = i

end function inbinom
