function nbinomcdf( k, r, p, ier )

!-----------------------------------------------------------------------
!
!     Computes the probability that a random variable distributed
!     according to the negative binomial distribution with R successes
!     and probability of success P, is less than or equal to K.
!
!     K     - Input . Value of the variable: number of        - Integer
!                     failures (K >= 0)
!     R     - Input . Number of successes (R > 0)             - Integer
!     P     - Input . Probability of success (0<= P <=1)      - Real
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                     3 = result out of limits in BETACDF
!
!     External functions called:
!       BETACDF (and DLGAMA if not in Fortran library)
!
!     Fortran functions called:
!       NONE
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: nbinomcdf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: p
   integer, intent(in) :: k, r
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

   real(kind=8), external :: betacdf

   real(kind=8), parameter :: zero=0.0_8, one=1.0_8
   real(kind=8) :: t

   !--------------------------------------------------------------------

   nbinomcdf = zero
   ier = 0

   !  Test for valid input arguments

   if ( r <= 0 .or. k < 0 .or. p <= zero .or. p > one ) then
      ier = 1
      return
   end if

   !  Special case

   if ( k == 0 .and. p == one ) then
      nbinomcdf = one
      return
   end if

   !  General case

   t = r
   nbinomcdf = betacdf( p, t, k+one, ier )

end function nbinomcdf
