function upperlimit( alpha, epsilon, u1, u2, thetamin, x, xp, n, ier, type )
   implicit none
   real(kind=8) :: upperlimit
   real(kind=8), intent(in) :: alpha, epsilon, u1, u2, thetamin
   integer, intent(in) :: x, xp, n
   character(len=*), intent(in) :: type
   integer, intent(out) :: ier

   real(kind=8), external :: sumdif
   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: zerolik=1.0e-13_8
   real(kind=8) :: dif, pq, u, umax, umin, ymax, ymin

   !--------------------------------------------------------------------

   ier = 0
   umin = u1
      ! x- < x+ ?
   pq = sumdif( u1, n, x, xp-1, dif, type )
   if ( dif > zero ) then
         ! Check it is worthwhile
      if ( thetamin >= u2 .or. sumdif(thetamin,n,x,xp,u,type) < alpha ) then
            ! Solve delta=0
         ymax = dif
         umax = u2
         pq = sumdif( umax, n, x, xp-1, ymin, type )
         u = umin
         do
            if ( abs(umax-umin) <= epsilon ) exit
            u = half*(umax+umin)   ! Halving
            pq = sumdif( u, n, x, xp-1, dif, type )
            if ( abs(dif) <= zerolik ) then
               exit
            else if ( dif > zero ) then
               umin = u
               ymax = dif
            else
               umax = u
               ymin = dif
            end if
            if ( abs(umax-umin) <= epsilon ) exit
            u = umax + ymin*(umax-umin)/(ymax-ymin) ! Interpolation
            pq = sumdif( u, n, x, xp-1, dif, type )
            if ( abs(dif) <= zerolik ) then
               exit
            else if ( dif > zero ) then
               umin = u
               ymax = dif
            else
               umax = u
               ymin = dif
            end if
         end do
         pq = sumdif( u, n, x, xp, dif, type )
         if ( pq < alpha ) then
            upperlimit = u
            return
         end if
         umin = u
         ymax = pq
      else
         ymax = sumdif( umin, n, x, xp, dif, type )
      end if
   else
      ymax = sumdif( umin, n, x, xp, dif, type )
   end if
      ! Solve xi=alpha
   umax = min(u2, thetamin)
   ymin = sumdif( umax, n, x, xp, dif, type )
   u = umin
   do
      if ( abs(umax-umin) <= epsilon ) exit
      u = half*(umax+umin)   ! Halving
      pq = sumdif( u, n, x, xp, dif, type )
      if ( abs(pq-alpha) <= zerolik ) then
         exit
      else if ( pq > alpha ) then
         umin = u
         ymax = pq
      else
         umax = u
         ymin = pq
      end if
      if ( abs(umax-umin) <= epsilon ) exit
      u = umax + (ymin-alpha)*(umax-umin)/(ymax-ymin) ! Interpolation
      pq = sumdif( u, n, x, xp, dif, type )
      if ( abs(pq-alpha) <= zerolik ) then
         exit
      else if ( pq > alpha ) then
         umin = u
         ymax = pq
      else
         umax = u
         ymin = pq
      end if
   end do

   upperlimit = u

end function upperlimit
